package puma.control;

/** Abstrakte Oberklasse aller "virtuellen" Buttons.
 * @author Sven Mller
 */
public abstract class FormButton {	
   /** Die aufzurufende Command-Instanz.
    */
   protected Command command;
   
   /** Speichert ein bestimmtes Command.    
    *
    * Die abzuleitende Unterklasse kennt den ihr zugehrigen Command-Typ.
    * Diesen speichert sie mit dieser Methode. Command verlangt im
    * Konstruktor den Parameter serverApp. Da Buttons i.d.R. dynamisch
    * generiert werden, ist dieser Parameter im eigenen Konstruktor noch
    * nicht bekannt und wird deshalb erst ber den Aufruf von execCommand(..)
    * nachgeliefert. Von dort wird auch storeCommand aufgerufen.
    *
    * @param serverApp  Ein Rckverweis auf die aufrufende Server-Instanz.
    */
   abstract public void storeCommand(ServerApplication serverApp);
   
   /** Liefert command zurck - Fr Testzwecke!
    *  @return Das dem Objekt zugeordnete Command command.
    */
   public final Command getCommand() {
      return command;
   }
   	
	/** Erzeugt das fr den Typ der Unterklasse definierte Command und fhrt es aus.
	 * @param in Objekt, das Input-Daten liefert.
    * @param out Objekt, das fr die Darstellung der Output-Daten sorgt.
    * @param serverApp  Ein Rckverweis auf die aufrufende Server-Instanz.    
    *
	 */
	public void execCommand(InType in, OutType out,
	                        ServerApplication serverApp) {
	   storeCommand(serverApp);
	   command.execute(in, out);
	}
}
