# ########################################################################
# ### schema.sql                                                       ###
# ### SQL-Datenbank-Schema fr die Publikationsverwaltung              ###
# ### Verantwortlich: Thomas Dreibholz, Dreibholz@bigfoot.com          ###
# ########################################################################



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ Publikations-Typen                                         @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### Publikations-Typen ##############################################
drop table if exists PUBLIKATIONS_TYPEN;
create table PUBLIKATIONS_TYPEN
(
   PUBL_TYP         char(20) not null,
   FELD_TYP         char(20) not null,
   FELD_NAME        char(20) not null,
   primary key (PUBL_TYP,FELD_TYP,FELD_NAME)
);
insert into PUBLIKATIONS_TYPEN values
   ("BUCH",       "INTEGER",     "SIGNATUR"),
   ("BUCH",       "STRING",      "TITEL"),
   ("BUCH",       "STRING",      "AUTOREN"),
   ("BUCH",       "INTEGER",     "JAHR"),
   ("BUCH",       "STRING",      "SPRACHE"),
   ("BUCH",       "STRING",      "ISBN"),
   ("BUCH",       "STRING",      "AUFLAGE"),
   ("BUCH",       "STRING",      "VERLAG"),
      
   ("TECHREP",    "INTEGER",     "SIGNATUR"),
   ("TECHREP",    "STRING",      "TITEL"),
   ("TECHREP",    "STRING",      "AUTOREN"),
   ("TECHREP",    "INTEGER",     "JAHR"),
   ("TECHREP",    "STRING",      "SPRACHE"),
   ("TECHREP",    "STRING",      "ISSN"),
   ("TECHREP",    "INTEGER",     "REPNR"),
   ("TECHREP",    "STRING",      "UNI"),

   ("ZEITSCHRIFT","INTEGER",     "SIGNATUR"),
   ("ZEITSCHRIFT","STRING",      "TITEL"),
   ("ZEITSCHRIFT","INTEGER",     "NUMMER"),
   ("ZEITSCHRIFT","INTEGER",     "JAHR"), 
   ("ZEITSCHRIFT","STRING",      "SPRACHE"),
   ("ZEITSCHRIFT","STRING",      "VERLAG");
optimize table PUBLIKATIONS_TYPEN;
      

# ###### Publikationstyp Buch ############################################
#
# Konvention: BUCH *mu* mindestens folgende Felder enthalten;
#             SIGNATUR (String)
#             TITEL (String)
#
drop table if exists BUCH;
create table BUCH
(
   SIGNATUR    char(20) not null,
   TITEL       char(80) not null,
   AUTOREN     char(60) not null,
   JAHR	       integer,
   SPRACHE     char(20),
   ISBN	       char(20),
   AUFLAGE     char(20),
   VERLAG      char(60),
   primary key (SIGNATUR),
   index TITEL_AUTOREN_INDEX (TITEL,AUTOREN),
   index TITEL_INDEX (TITEL)
);


# ###### Publikationstyp TechRep #########################################
#
# Konvention: TECHREP *mu* mindestens folgende Felder enthalten;
#             SIGNATUR (String)
#             TITEL (String)
#
drop table if exists TECHREP;
create table TECHREP
(
   SIGNATUR    char(20) not null,
   TITEL       char(80) not null,
   AUTOREN     char(60) not null,
   JAHR	       integer,
   SPRACHE     char(20),
   ISSN	       char(20),
   REPNR       integer,
   UNI         char(60) not null,
   primary key (SIGNATUR),
   index TITEL_AUTOREN_INDEX (TITEL,AUTOREN),
   index TITEL_UNI_INDEX (TITEL,UNI),
   index TITEL_INDEX (TITEL)         
);


# ###### Publikationstyp Zeitschrift #####################################
#
# Konvention: ZEITSCHRIFT *mu* mindestens folgende Felder enthalten;
#             SIGNATUR (String)
#             TITEL (String)
#
drop table if exists ZEITSCHRIFT;
create table ZEITSCHRIFT
(
   SIGNATUR    char(20) not null,
   TITEL       char(80) not null,
   NUMMER      integer  not null,
   JAHR	       integer,
   SPRACHE     char(20),
   VERLAG      char(60),
   primary key (SIGNATUR),
   index TITEL_NUMMER_INDEX (TITEL,NUMMER),
   index TITEL_INDEX (TITEL)         
);



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ Masken                                                     @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### Masken ##########################################################
drop table if exists MASKEN;
create table MASKEN
(
   MASKE            char(20) not null,
   PUBL_TYP         char(20) not null,
   SICHTBAR         char(20) not null,
   primary key (MASKE,PUBL_TYP,SICHTBAR)
);
insert into MASKEN values
   ("SUCHMASKE","BUCH",       "SIGNATUR"),
   ("SUCHMASKE","BUCH",       "TITEL"),
   ("SUCHMASKE","BUCH",       "AUTOREN"),
   ("SUCHMASKE","BUCH",       "JAHR"),
   ("SUCHMASKE","BUCH",       "SPRACHE"),
   ("SUCHMASKE","BUCH",       "ISBN"),
   ("SUCHMASKE","BUCH",       "VERLAG"),
   
   ("SUCHMASKE","TECHREP",    "SIGNATUR"),
   ("SUCHMASKE","TECHREP",    "TITEL"),
   ("SUCHMASKE","TECHREP",    "AUTOREN"),
   ("SUCHMASKE","TECHREP",    "JAHR"),
   ("SUCHMASKE","TECHREP",    "SPRACHE"),
   ("SUCHMASKE","TECHREP",    "ISSN"),
   ("SUCHMASKE","TECHREP",    "UNI"),
   
   ("SUCHMASKE","ZEITSCHRIFT","SIGNATUR"),
   ("SUCHMASKE","ZEITSCHRIFT","TITEL"),
   ("SUCHMASKE","ZEITSCHRIFT","NUMMER"),
   ("SUCHMASKE","ZEITSCHRIFT","JAHR"),
   ("SUCHMASKE","ZEITSCHRIFT","SPRACHE"),
   ("SUCHMASKE","ZEITSCHRIFT","VERLAG"),

   ("ERGEBNIS","BUCH",       "SIGNATUR"),
   ("ERGEBNIS","BUCH",       "TITEL"),
   ("ERGEBNIS","BUCH",       "AUTOREN"),
   ("ERGEBNIS","BUCH",       "JAHR"),
   ("ERGEBNIS","BUCH",       "SPRACHE"),
   ("ERGEBNIS","BUCH",       "ISBN"),
   ("ERGEBNIS","BUCH",       "AUFLAGE"),
   ("ERGEBNIS","BUCH",       "VERLAG"),
   
   ("ERGEBNIS","TECHREP",    "SIGNATUR"),
   ("ERGEBNIS","TECHREP",    "TITEL"),
   ("ERGEBNIS","TECHREP",    "AUTOREN"),
   ("ERGEBNIS","TECHREP",    "JAHR"),
   ("ERGEBNIS","TECHREP",    "SPRACHE"),
   ("ERGEBNIS","TECHREP",    "ISSN"),
   ("ERGEBNIS","TECHREP",    "REFNR"),
   ("ERGEBNIS","TECHREP",    "UNI"),
   
   ("ERGEBNIS","ZEITSCHRIFT","SIGNATUR"),
   ("ERGEBNIS","ZEITSCHRIFT","TITEL"),
   ("ERGEBNIS","ZEITSCHRIFT","NUMMER"),
   ("ERGEBNIS","ZEITSCHRIFT","JAHR"),
   ("ERGEBNIS","ZEITSCHRIFT","SPRACHE"),
   ("ERGEBNIS","ZEITSCHRIFT","VERLAG"),

   ("ALISTE",   "BUCH",       "SIGNATUR"),
   ("ALISTE",   "BUCH",       "TITEL"),
   ("ALISTE",   "BUCH",       "AUTOREN"),
   ("ALISTE",   "BUCH",       "ISBN"),
   
   ("ALISTE",   "TECHREP",    "SIGNATUR"),
   ("ALISTE",   "TECHREP",    "TITEL"),
   ("ALISTE",   "TECHREP",    "AUTOREN"),
   ("ALISTE",   "TECHREP",    "ISSN"),
   
   ("ALISTE",   "ZEITSCHRIFT","SIGNATUR"),
   ("ALISTE",   "ZEITSCHRIFT","TITEL"),
   ("ALISTE",   "ZEITSCHRIFT","NUMMER"),
   ("ALISTE",   "ZEITSCHRIFT","JAHR"),

   ("ALISTE",   "MITARBEITER","VORNAME"),
   ("ALISTE",   "MITARBEITER","NAME");
optimize table MASKEN;
   


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ Instanzen                                                  @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### Instanz-Felder ##################################################
drop table if exists INSTANZ_FELDER;
create table INSTANZ_FELDER
(
   FELD_TYP         char(20) not null,
   FELD_NAME        char(20) not null
);
insert into INSTANZ_FELDER values
   ("STRING", "INV_NR"),
   ("STRING", "SIGNATUR"),
   ("STRING", "STATUS"),
   ("INTEGER","PERSON_ID");
optimize table INSTANZ_FELDER;


# ###### Instanz (Inventar-Objekt) #######################################
#
# Konvention: INSTANZ *mu* mindestens folgende Felder enthalten;
#             INV_NR (String)
#             SIGNATUR (String)
#             STATUS (String)
#             PERSON_ID (Integer)
#
drop table if exists INSTANZ;
create table INSTANZ
(
   INV_NR      char(20) not null,
   SIGNATUR    char(20) not null,
   STATUS      char(20) not null,
   PERSON_ID   integer,
   primary key (INV_NR),
   foreign key (PERSON_ID) references PERSON (PERSON_ID),
   index SIG_INV_INDEX (SIGNATUR,INV_NR)
);


# ###### Mgliche Instanz-Status-Werte ###################################
#
# Konvention: INSTANZ_STATUS_TYPEN *mu* mindestens folgende Werte enthalten;
#             VERFUEGBAR, AUSGELIEHEN, VERSCHOLLEN
#
drop table if exists INSTANZ_STATUS_TYPEN;
create table INSTANZ_STATUS_TYPEN
(
   STATUS      char(20) not null,
   primary key (STATUS)
);
insert into INSTANZ_STATUS_TYPEN values
   ("VERFUEGBAR"),
   ("AUSGELIEHEN"),
   ("VERSCHOLLEN");
optimize table INSTANZ_STATUS_TYPEN;
      


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ Personen                                                   @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### Person-Felder ###################################################
drop table if exists PERSON_FELDER;
create table PERSON_FELDER
(
   FELD_TYP         char(20) not null,
   FELD_NAME        char(20) not null
);
insert into PERSON_FELDER values
   ("INTEGER", "PERSON_ID"),
   ("STRING",  "VORNAME"),
   ("STRING",  "NAME"),
   ("STRING",  "PASSWORT"),
   ("STRING",  "TELEFON"),
   ("STRING",  "TELEFAX"),
   ("STRING",  "EMAIL"),
   ("STRING",  "RAUM");


# ###### Person ##########################################################
drop table if exists PERSON;
create table PERSON
(
   PERSON_ID   integer  not null,
   VORNAME     char(40) not null,
   NAME	       char(40) not null,
   PASSWORT    char(12),
   TELEFON     char(20),
   TELEFAX     char(20),
   EMAIL       char(40),
   RAUM        char(10),
   primary key (PERSON_ID),
   index VN_NN_INDEX (VORNAME,NAME)
);



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ Zugriffsrechte                                             @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### Standard-Zugriffsrechts #########################################
#
# Mgliche Zugriffsrechte:
#
# AUSLEIH
# AUSLEIHLISTE
# DETAIL
# RUECKGABE
# WEITERGABE
# SUCHEN
# PUBL_LOESCHEN
# PUBL_HINZUFUEGEN
# PUBL_AENDERN
# PERSON_LOESCHEN
# PERSON_HINZUFUEGEN
# PERSON_AENDERN
# RECHTE_AENDERN
# HISTORY_SEHEN
# HISTORY_LOESCHEN
#
drop table if exists STANDARD_ZGR;
create table STANDARD_ZGR
(
   RECHT       char(20) not null,
   primary key (RECHT)
);
insert into STANDARD_ZGR values
   ("SUCHEN"),
   ("AUSLEIHLISTE"),
   ("DETAIL");
optimize table STANDARD_ZGR;


# ###### Zugriffsrechte fr Personen #####################################
drop table if exists ZUGRIFFSRECHTE;
create table ZUGRIFFSRECHTE
(
   PERSON_ID   integer not null,
   RECHT       char(20) not null,
   primary key (PERSON_ID,RECHT),
   foreign key (PERSON_ID) references PERSON (PERSON_ID)
);
 


# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@ History                                                    @@@@@@
# @@@@@@                                                            @@@@@@
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# ###### History-Felder ##################################################
drop table if exists HISTORY_FELDER;
create table HISTORY_FELDER
(
   FELD_TYP         char(20) not null,
   FELD_NAME        char(20) not null
);
insert into HISTORY_FELDER values
   ("STRING",  "INV_NR"),
   ("STRING",  "ZEITPUNKT"),
   ("STRING",  "STATUS"),
   ("INTEGER", "PERSON_ID");
optimize table HISTORY_FELDER;
            
 
# ###### History #########################################################
#
# Konvention: HISTORY *mu* mindestens folgende Felder enthalten;
#             INV_NR (String)
#             ZEITPUNKT (String)
#             STATUS (String)
#             PERSON_ID (Integer)
#
drop table if exists HISTORY;
create table HISTORY
(
   INV_NR      char(20) not null,
   ZEITPUNKT   timestamp not null,
   STATUS      char(20) not null,
   PERSON_ID   integer,
   primary key (INV_NR,ZEITPUNKT,STATUS),
   foreign key (INV_NR) references INSTANZ (INV_NR),
   foreign key (PERSON_ID) references PERSON (PERSON_ID)
);
